#include "AudioAmp2_demo_objects.h"
#include "AudioAmp2_demo_resources.h"
#include "built_in.h"


// TFT module connections
sbit TFT_BLED at GPIOE_ODR.B9;
sbit TFT_CS at GPIOE_ODR.B15;
unsigned int TFT_DataPort at GPIOE_ODR;
sbit TFT_RD at GPIOE_ODR.B10;
sbit TFT_RS at GPIOE_ODR.B12;
sbit TFT_RST at GPIOE_ODR.B8;
sbit TFT_WR at GPIOE_ODR.B11;
// End TFT module connections

// Touch Panel module connections
sbit DriveX_Left at GPIOB_ODR.B1;
sbit DriveX_Right at GPIOB_ODR.B8;
sbit DriveY_Up at GPIOB_ODR.B9;
sbit DriveY_Down at GPIOB_ODR.B0;
// End Touch Panel module connections

// Global variables
unsigned int Xcoord, Ycoord;
const ADC_THRESHOLD = 1500;
char PenDown;
void *PressedObject;
int PressedObjectType;
unsigned int caption_length, caption_height;
unsigned int display_width, display_height;

int _object_count;
unsigned short object_pressed;
TButton_Round *local_round_button;
TButton_Round *exec_round_button;
int round_button_order;
TImage *local_image;
TImage *exec_image;
int image_order;


void Init_ADC() {
  ADC_Set_Input_Channel(_ADC_CHANNEL_8 | _ADC_CHANNEL_9);
  ADC1_Init();
  Delay_ms(100);
}
static void InitializeTouchPanel() {
  Init_ADC();
  TFT_Init_ILI9341_8bit(320, 240);

  TP_TFT_Init(320, 240, 8, 9);                                  // Initialize touch panel
  TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold

  PenDown = 0;
  PressedObject = 0;
  PressedObjectType = -1;
}

void Calibrate() {
  TFT_Set_Pen(CL_WHITE, 3);
  TFT_Set_Font(TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);
  TFT_Write_Text("Touch selected corners for calibration", 50, 80);
  TFT_Line(315, 239, 319, 239);
  TFT_Line(309, 229, 319, 239);
  TFT_Line(319, 234, 319, 239);
  TFT_Write_Text("first here", 210, 220);

  TP_TFT_Calibrate_Min();                      // Calibration of TP minimum
  Delay_ms(500);

  TFT_Set_Pen(CL_BLACK, 3);
  TFT_Set_Font(TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  TFT_Line(315, 239, 319, 239);
  TFT_Line(309, 229, 319, 239);
  TFT_Line(319, 234, 319, 239);
  TFT_Write_Text("first here", 210, 220);

  TFT_Set_Pen(CL_WHITE, 3);
  TFT_Set_Font(TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);
  TFT_Write_Text("now here ", 20, 5);
  TFT_Line(0, 0, 5, 0);
  TFT_Line(0, 0, 0, 5);
  TFT_Line(0, 0, 10, 10);

  TP_TFT_Calibrate_Max();                       // Calibration of TP maximum
  Delay_ms(500);
}


/////////////////////////
  TScreen*  CurrentScreen;

  TScreen                Screen1;
  TImage               Image1;
  TButton_Round          ButtonRound1;
char ButtonRound1_Caption[5] = "20db";

  TButton_Round          ButtonRound2;
char ButtonRound2_Caption[5] = "26db";

  TButton_Round          ButtonRound3;
char ButtonRound3_Caption[5] = "30db";

  TButton_Round          ButtonRound4;
char ButtonRound4_Caption[5] = "32db";

  TButton_Round          ButtonRound5;
char ButtonRound5_Caption[12] = "MUTE/UNMUTE";

  TButton_Round          ButtonRound6;
char ButtonRound6_Caption[8] = "STANDBY";

  TButton_Round          * const code Screen1_Buttons_Round[6]=
         {
         &ButtonRound1,        
         &ButtonRound2,        
         &ButtonRound3,        
         &ButtonRound4,        
         &ButtonRound5,        
         &ButtonRound6         
         };
  TImage                 * const code Screen1_Images[1]=
         {
         &Image1               
         };



static void InitializeObjects() {
  Screen1.Color                     = 0x5AEB;
  Screen1.Width                     = 320;
  Screen1.Height                    = 240;
  Screen1.Buttons_RoundCount        = 6;
  Screen1.Buttons_Round             = Screen1_Buttons_Round;
  Screen1.ImagesCount               = 1;
  Screen1.Images                    = Screen1_Images;
  Screen1.ObjectsCount              = 7;


  Image1.OwnerScreen     = &Screen1;
  Image1.Order           = 0;
  Image1.Left            = 2;
  Image1.Top             = 0;
  Image1.Width           = 320;
  Image1.Height          = 240;
  Image1.Picture_Type    = 0;
  Image1.Picture_Ratio   = 1;
  Image1.Picture_Name    = AudioAmp2click_bmp;
  Image1.Visible         = 1;
  Image1.Active          = 1;
  Image1.OnUpPtr         = 0;
  Image1.OnDownPtr       = 0;
  Image1.OnClickPtr      = 0;
  Image1.OnPressPtr      = 0;

  ButtonRound1.OwnerScreen     = &Screen1;
  ButtonRound1.Order           = 1;
  ButtonRound1.Left            = 128;
  ButtonRound1.Top             = 25;
  ButtonRound1.Width           = 79;
  ButtonRound1.Height          = 30;
  ButtonRound1.Pen_Width       = 0;
  ButtonRound1.Pen_Color       = 0x0000;
  ButtonRound1.Visible         = 1;
  ButtonRound1.Active          = 1;
  ButtonRound1.Transparent     = 1;
  ButtonRound1.Caption         = ButtonRound1_Caption;
  ButtonRound1.TextAlign       = _taCenter;
  ButtonRound1.TextAlignVertical= _tavMiddle;
  ButtonRound1.FontName        = Source_Sans_Pro_Semibold14x20_Regular;
  ButtonRound1.PressColEnabled = 1;
  ButtonRound1.Font_Color      = 0xFFFF;
  ButtonRound1.VerticalText    = 0;
  ButtonRound1.Gradient        = 1;
  ButtonRound1.Gradient_Orientation = 0;
  ButtonRound1.Gradient_Start_Color = 0x632C;
  ButtonRound1.Gradient_End_Color = 0x632C;
  ButtonRound1.Color           = 0xC618;
  ButtonRound1.Press_Color     = 0xE71C;
  ButtonRound1.Corner_Radius   = 5;
  ButtonRound1.OnUpPtr         = 0;
  ButtonRound1.OnDownPtr       = 0;
  ButtonRound1.OnClickPtr      = ButtonRound1OnClick;
  ButtonRound1.OnPressPtr      = 0;

  ButtonRound2.OwnerScreen     = &Screen1;
  ButtonRound2.Order           = 2;
  ButtonRound2.Left            = 218;
  ButtonRound2.Top             = 25;
  ButtonRound2.Width           = 79;
  ButtonRound2.Height          = 30;
  ButtonRound2.Pen_Width       = 0;
  ButtonRound2.Pen_Color       = 0x0000;
  ButtonRound2.Visible         = 1;
  ButtonRound2.Active          = 1;
  ButtonRound2.Transparent     = 1;
  ButtonRound2.Caption         = ButtonRound2_Caption;
  ButtonRound2.TextAlign       = _taCenter;
  ButtonRound2.TextAlignVertical= _tavMiddle;
  ButtonRound2.FontName        = Source_Sans_Pro_Semibold14x20_Regular;
  ButtonRound2.PressColEnabled = 1;
  ButtonRound2.Font_Color      = 0xFFFF;
  ButtonRound2.VerticalText    = 0;
  ButtonRound2.Gradient        = 1;
  ButtonRound2.Gradient_Orientation = 0;
  ButtonRound2.Gradient_Start_Color = 0x632C;
  ButtonRound2.Gradient_End_Color = 0x632C;
  ButtonRound2.Color           = 0xC618;
  ButtonRound2.Press_Color     = 0xE71C;
  ButtonRound2.Corner_Radius   = 5;
  ButtonRound2.OnUpPtr         = 0;
  ButtonRound2.OnDownPtr       = 0;
  ButtonRound2.OnClickPtr      = ButtonRound2OnClick;
  ButtonRound2.OnPressPtr      = 0;

  ButtonRound3.OwnerScreen     = &Screen1;
  ButtonRound3.Order           = 3;
  ButtonRound3.Left            = 128;
  ButtonRound3.Top             = 61;
  ButtonRound3.Width           = 79;
  ButtonRound3.Height          = 30;
  ButtonRound3.Pen_Width       = 0;
  ButtonRound3.Pen_Color       = 0x0000;
  ButtonRound3.Visible         = 1;
  ButtonRound3.Active          = 1;
  ButtonRound3.Transparent     = 1;
  ButtonRound3.Caption         = ButtonRound3_Caption;
  ButtonRound3.TextAlign       = _taCenter;
  ButtonRound3.TextAlignVertical= _tavMiddle;
  ButtonRound3.FontName        = Source_Sans_Pro_Semibold14x20_Regular;
  ButtonRound3.PressColEnabled = 1;
  ButtonRound3.Font_Color      = 0xFFFF;
  ButtonRound3.VerticalText    = 0;
  ButtonRound3.Gradient        = 1;
  ButtonRound3.Gradient_Orientation = 0;
  ButtonRound3.Gradient_Start_Color = 0x632C;
  ButtonRound3.Gradient_End_Color = 0x632C;
  ButtonRound3.Color           = 0xC618;
  ButtonRound3.Press_Color     = 0xE71C;
  ButtonRound3.Corner_Radius   = 5;
  ButtonRound3.OnUpPtr         = 0;
  ButtonRound3.OnDownPtr       = 0;
  ButtonRound3.OnClickPtr      = ButtonRound3OnClick;
  ButtonRound3.OnPressPtr      = 0;

  ButtonRound4.OwnerScreen     = &Screen1;
  ButtonRound4.Order           = 4;
  ButtonRound4.Left            = 219;
  ButtonRound4.Top             = 61;
  ButtonRound4.Width           = 79;
  ButtonRound4.Height          = 30;
  ButtonRound4.Pen_Width       = 0;
  ButtonRound4.Pen_Color       = 0x0000;
  ButtonRound4.Visible         = 1;
  ButtonRound4.Active          = 1;
  ButtonRound4.Transparent     = 1;
  ButtonRound4.Caption         = ButtonRound4_Caption;
  ButtonRound4.TextAlign       = _taCenter;
  ButtonRound4.TextAlignVertical= _tavMiddle;
  ButtonRound4.FontName        = Source_Sans_Pro_Semibold14x20_Regular;
  ButtonRound4.PressColEnabled = 1;
  ButtonRound4.Font_Color      = 0xFFFF;
  ButtonRound4.VerticalText    = 0;
  ButtonRound4.Gradient        = 1;
  ButtonRound4.Gradient_Orientation = 0;
  ButtonRound4.Gradient_Start_Color = 0x632C;
  ButtonRound4.Gradient_End_Color = 0x632C;
  ButtonRound4.Color           = 0xC618;
  ButtonRound4.Press_Color     = 0xE71C;
  ButtonRound4.Corner_Radius   = 5;
  ButtonRound4.OnUpPtr         = 0;
  ButtonRound4.OnDownPtr       = 0;
  ButtonRound4.OnClickPtr      = ButtonRound4OnClick;
  ButtonRound4.OnPressPtr      = 0;

  ButtonRound5.OwnerScreen     = &Screen1;
  ButtonRound5.Order           = 5;
  ButtonRound5.Left            = 128;
  ButtonRound5.Top             = 97;
  ButtonRound5.Width           = 169;
  ButtonRound5.Height          = 30;
  ButtonRound5.Pen_Width       = 0;
  ButtonRound5.Pen_Color       = 0x0000;
  ButtonRound5.Visible         = 1;
  ButtonRound5.Active          = 1;
  ButtonRound5.Transparent     = 1;
  ButtonRound5.Caption         = ButtonRound5_Caption;
  ButtonRound5.TextAlign       = _taCenter;
  ButtonRound5.TextAlignVertical= _tavMiddle;
  ButtonRound5.FontName        = Source_Sans_Pro_Semibold14x20_Regular;
  ButtonRound5.PressColEnabled = 1;
  ButtonRound5.Font_Color      = 0xFFFF;
  ButtonRound5.VerticalText    = 0;
  ButtonRound5.Gradient        = 1;
  ButtonRound5.Gradient_Orientation = 0;
  ButtonRound5.Gradient_Start_Color = 0x632C;
  ButtonRound5.Gradient_End_Color = 0x632C;
  ButtonRound5.Color           = 0xC618;
  ButtonRound5.Press_Color     = 0xE71C;
  ButtonRound5.Corner_Radius   = 5;
  ButtonRound5.OnUpPtr         = 0;
  ButtonRound5.OnDownPtr       = 0;
  ButtonRound5.OnClickPtr      = ButtonRound5OnClick;
  ButtonRound5.OnPressPtr      = 0;

  ButtonRound6.OwnerScreen     = &Screen1;
  ButtonRound6.Order           = 6;
  ButtonRound6.Left            = 128;
  ButtonRound6.Top             = 133;
  ButtonRound6.Width           = 169;
  ButtonRound6.Height          = 30;
  ButtonRound6.Pen_Width       = 0;
  ButtonRound6.Pen_Color       = 0x0000;
  ButtonRound6.Visible         = 1;
  ButtonRound6.Active          = 1;
  ButtonRound6.Transparent     = 1;
  ButtonRound6.Caption         = ButtonRound6_Caption;
  ButtonRound6.TextAlign       = _taCenter;
  ButtonRound6.TextAlignVertical= _tavMiddle;
  ButtonRound6.FontName        = Source_Sans_Pro_Semibold14x20_Regular;
  ButtonRound6.PressColEnabled = 1;
  ButtonRound6.Font_Color      = 0xFFFF;
  ButtonRound6.VerticalText    = 0;
  ButtonRound6.Gradient        = 1;
  ButtonRound6.Gradient_Orientation = 0;
  ButtonRound6.Gradient_Start_Color = 0x632C;
  ButtonRound6.Gradient_End_Color = 0x632C;
  ButtonRound6.Color           = 0xC618;
  ButtonRound6.Press_Color     = 0xE71C;
  ButtonRound6.Corner_Radius   = 5;
  ButtonRound6.OnUpPtr         = 0;
  ButtonRound6.OnDownPtr       = 0;
  ButtonRound6.OnClickPtr      = ButtonRound6OnClick;
  ButtonRound6.OnPressPtr      = 0;
}

static char IsInsideObject (unsigned int X, unsigned int Y, unsigned int Left, unsigned int Top, unsigned int Width, unsigned int Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetRoundButton(index)         CurrentScreen->Buttons_Round[index]
#define GetImage(index)               CurrentScreen->Images[index]


void DrawRoundButton(TButton_Round *Around_button) {
unsigned int ALeft, ATop;
  if (Around_button->Visible != 0) {
    if (object_pressed == 1) {
      object_pressed = 0;
      TFT_Set_Brush(Around_button->Transparent, Around_button->Press_Color, Around_button->Gradient, Around_button->Gradient_Orientation,
                    Around_button->Gradient_End_Color, Around_button->Gradient_Start_Color);
    }
    else {
      TFT_Set_Brush(Around_button->Transparent, Around_button->Color, Around_button->Gradient, Around_button->Gradient_Orientation,
                    Around_button->Gradient_Start_Color, Around_button->Gradient_End_Color);
    }
    TFT_Set_Pen(Around_button->Pen_Color, Around_button->Pen_Width);
    TFT_Rectangle_Round_Edges(Around_button->Left + 1, Around_button->Top + 1,
      Around_button->Left + Around_button->Width - 2,
      Around_button->Top + Around_button->Height - 2, Around_button->Corner_Radius);
    if (Around_button->VerticalText != 0)
      TFT_Set_Font(Around_button->FontName, Around_button->Font_Color, FO_VERTICAL_COLUMN);
    else
      TFT_Set_Font(Around_button->FontName, Around_button->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text_Return_Pos(Around_button->Caption, Around_button->Left, Around_button->Top);
    if (Around_button->TextAlign == _taLeft)
      ALeft = Around_button->Left + 4;
    else if (Around_button->TextAlign == _taCenter)
      ALeft = Around_button->Left + (Around_button->Width - caption_length) / 2;
    else if (Around_button->TextAlign == _taRight)
      ALeft = Around_button->Left + Around_button->Width - caption_length - 4;

    if (Around_button->TextAlignVertical == _tavTop)
      ATop = Around_button->Top + 3;
    else if (Around_button->TextAlignVertical == _tavMiddle)
      ATop = Around_button->Top + (Around_button->Height - caption_height) / 2;
    else if (Around_button->TextAlignVertical == _tavBottom)
      ATop  = Around_button->Top + Around_button->Height - caption_height - 4;

    TFT_Write_Text(Around_button->Caption, ALeft, ATop);
  }
}

void DrawImage(TImage *AImage) {
  if (AImage->Visible != 0) {
    TFT_Image(AImage->Left, AImage->Top, AImage->Picture_Name, AImage->Picture_Ratio);
  }
}

void DrawScreen(TScreen *aScreen) {
 unsigned int order;
  unsigned short round_button_idx;
  TButton_Round *local_round_button;
  unsigned short image_idx;
  TImage *local_image;
  char save_bled;

  object_pressed = 0;
  order = 0;
  round_button_idx = 0;
  image_idx = 0;
  CurrentScreen = aScreen;

  if ((display_width != CurrentScreen->Width) || (display_height != CurrentScreen->Height)) {
    save_bled = TFT_BLED;
    TFT_BLED           = 0;
    TFT_Init_ILI9341_8bit(CurrentScreen->Width, CurrentScreen->Height);
    TP_TFT_Init(CurrentScreen->Width, CurrentScreen->Height, 8, 9);                                  // Initialize touch panel
    TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold
    TFT_Fill_Screen(CurrentScreen->Color);
    display_width = CurrentScreen->Width;
    display_height = CurrentScreen->Height;
    TFT_BLED           = save_bled;
  }
  else
    TFT_Fill_Screen(CurrentScreen->Color);


  while (order < CurrentScreen->ObjectsCount) {
    if (round_button_idx < CurrentScreen->Buttons_RoundCount) {
      local_round_button = GetRoundButton(round_button_idx);
      if (order == local_round_button->Order) {
        round_button_idx++;
        order++;
        DrawRoundButton(local_round_button);
      }
    }

    if (image_idx < CurrentScreen->ImagesCount) {
      local_image = GetImage(image_idx);
      if (order == local_image->Order) {
        image_idx++;
        order++;
        DrawImage(local_image);
      }
    }

  }
}

void Get_Object(unsigned int X, unsigned int Y) {
  round_button_order  = -1;
  image_order         = -1;
  //  Buttons with Round Edges
  for ( _object_count = 0 ; _object_count < CurrentScreen->Buttons_RoundCount ; _object_count++ ) {
    local_round_button = GetRoundButton(_object_count);
    if (local_round_button->Active != 0) {
      if (IsInsideObject(X, Y, local_round_button->Left, local_round_button->Top,
                         local_round_button->Width, local_round_button->Height) == 1) {
        round_button_order = local_round_button->Order;
        exec_round_button = local_round_button;
      }
    }
  }

  //  Images
  for ( _object_count = 0 ; _object_count < CurrentScreen->ImagesCount ; _object_count++ ) {
    local_image = GetImage(_object_count);
    if (local_image->Active != 0) {
      if (IsInsideObject(X, Y, local_image->Left, local_image->Top,
                         local_image->Width, local_image->Height) == 1) {
        image_order = local_image->Order;
        exec_image = local_image;
      }
    }
  }

  _object_count = -1;
  if (round_button_order >  _object_count )
    _object_count = round_button_order;
  if (image_order >  _object_count )
    _object_count = image_order;
}


void Process_TP_Press(unsigned int X, unsigned int Y) {
  exec_round_button   = 0;
  exec_image          = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == round_button_order) {
      if (exec_round_button->Active != 0) {
        if (exec_round_button->OnPressPtr != 0) {
          exec_round_button->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == image_order) {
      if (exec_image->Active != 0) {
        if (exec_image->OnPressPtr != 0) {
          exec_image->OnPressPtr();
          return;
        }
      }
    }

  }
}

void Process_TP_Up(unsigned int X, unsigned int Y) {

  switch (PressedObjectType) {
    // Round Button
    case 1: {
      if (PressedObject != 0) {
        exec_round_button = (TButton_Round*)PressedObject;
        if ((exec_round_button->PressColEnabled != 0) && (exec_round_button->OwnerScreen == CurrentScreen)) {
          DrawRoundButton(exec_round_button);
        }
        break;
      }
      break;
    }
  }

  exec_image          = 0;

  Get_Object(X, Y);


  if (_object_count != -1) {
  // Buttons with Round Edges
    if (_object_count == round_button_order) {
      if (exec_round_button->Active != 0) {
        if (exec_round_button->OnUpPtr != 0)
          exec_round_button->OnUpPtr();
        if (PressedObject == (void *)exec_round_button)
          if (exec_round_button->OnClickPtr != 0)
            exec_round_button->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Images
    if (_object_count == image_order) {
      if (exec_image->Active != 0) {
        if (exec_image->OnUpPtr != 0)
          exec_image->OnUpPtr();
        if (PressedObject == (void *)exec_image)
          if (exec_image->OnClickPtr != 0)
            exec_image->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  }
  PressedObject = 0;
  PressedObjectType = -1;
}

void Process_TP_Down(unsigned int X, unsigned int Y) {

  object_pressed      = 0;
  exec_round_button   = 0;
  exec_image          = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == round_button_order) {
      if (exec_round_button->Active != 0) {
        if (exec_round_button->PressColEnabled != 0) {
          object_pressed = 1;
          DrawRoundButton(exec_round_button);
        }
        PressedObject = (void *)exec_round_button;
        PressedObjectType = 1;
        if (exec_round_button->OnDownPtr != 0) {
          exec_round_button->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == image_order) {
      if (exec_image->Active != 0) {
        PressedObject = (void *)exec_image;
        PressedObjectType = 3;
        if (exec_image->OnDownPtr != 0) {
          exec_image->OnDownPtr();
          return;
        }
      }
    }

  }
}

void Check_TP() {
  if (TP_TFT_Press_Detect()) {
    if (TP_TFT_Get_Coordinates(&Xcoord, &Ycoord) == 0) {
    // After a PRESS is detected read X-Y and convert it to Display dimensions space
      Process_TP_Press(Xcoord, Ycoord);
      if (PenDown == 0) {
        PenDown = 1;
        Process_TP_Down(Xcoord, Ycoord);
      }
    }
  }
  else if (PenDown == 1) {
    PenDown = 0;
    Process_TP_Up(Xcoord, Ycoord);
  }
}

void Init_MCU() {
  GPIO_Digital_Output(&GPIOE_BASE, _GPIO_PINMASK_9);
  TFT_BLED = 1;
  TFT_Set_Default_Mode();
  TP_TFT_Set_Default_Mode();
}

void Start_TP() {
  Init_MCU();

  InitializeTouchPanel();

  Delay_ms(1000);
  TFT_Fill_Screen(0);
  Calibrate();
  TFT_Fill_Screen(0);

  InitializeObjects();
  display_width = Screen1.Width;
  display_height = Screen1.Height;
  DrawScreen(&Screen1);
}